@@ -66,13 +66,20 @@ class FolderFileChooserDialog(Gtk.Dialog):
 class FileObject():
 
     def __init__(self, path_or_uri):
-        if "://" in path_or_uri:
-            self.gfile = Gio.File.new_for_uri(path_or_uri)
-        else:
-            self.gfile = Gio.File.new_for_path(path_or_uri)
 
+        self.gfile = self.create_gfile(path_or_uri)
         self._update_info()
 
+    def create_gfile(self, path_or_uri):
+        gfile = None
+
+        if "://" in path_or_uri:
+            gfile = Gio.File.new_for_uri(path_or_uri)
+        else:
+            gfile = Gio.File.new_for_path(path_or_uri)
+
+        return gfile
+
     def _update_info(self):
         self.info = None
         self.uri = self.gfile.get_uri()
@@ -105,8 +112,15 @@ class FileObject():
         self.is_valid = True
 
     def rename(self, new_name):
-        # this can fail, our caller will catch
-        new_gfile = self.gfile.set_display_name(new_name, None)
+        backup_gfile = self.gfile.dup()
+        try:
+            new_gfile = self.gfile.set_display_name(new_name, None)
+        except:
+            # This can fail - make sure the GFile is still in a pre-fail state
+            # (not sure this is necessary), then let the caller handle telling
+            # the user.
+            self.gfile = backup_gfile
+            raise
 
         self.gfile = new_gfile
         self._update_info()
@@ -396,6 +410,21 @@ class MainWindow():
     def on_close_button(self, widget):
         self.application.quit()
 
+    def report_os_error(self, file_obj, new_path, error):
+        message = ""
+        # Gio's error prints the too-long filename as part of its
+        # error message, which looks really bad. Maybe we'll need
+        # to add more exceptions here for other error codes.
+        if Gio.IOErrorEnum(error.code) == Gio.IOErrorEnum.FILENAME_TOO_LONG:
+            message = _("Unable to rename '%s': File name too long") \
+                % file_obj.get_path_or_uri_for_display()
+        else:
+            message = _("Unable to rename '%s': %s") \
+                % (file_obj.get_path_or_uri_for_display(), error.message)
+
+        self.error_label.set_text(message)
+        self.infobar.show()
+
     def on_rename_button(self, widget):
         iters = []
         iter = self.model.get_iter_first()
@@ -421,17 +450,18 @@ class MainWindow():
         #     print(i[0].gfile.get_uri())
 
         for tup in rename_list:
-            try:
                 iter, file_obj, name, new_name = tup
                 if new_name != name:
-                    old_uri = file_obj.uri
-                    if file_obj.rename(new_name):
-                        self.uris.remove(old_uri)
-                        self.uris.append(file_obj.uri)
-                        self.model.set_value(iter, COL_NAME, new_name)
-                        print("Renamed %s --> %s" % (name, new_name))
-            except Exception as e:
-                print(e)
+                    try:
+                        old_uri = file_obj.uri
+                        # print("Renaming %s --> %s" % (file_obj.get_path_or_uri_for_display(), new_name))
+                        if file_obj.rename(new_name):
+                            self.uris.remove(old_uri)
+                            self.uris.append(file_obj.uri)
+                            self.model.set_value(iter, COL_NAME, new_name)
+                    except GLib.Error as e:
+                        self.report_os_error(file_obj, new_name, e)
+                        break
 
         self.rename_button.set_sensitive(False)
 
@@ -509,16 +539,38 @@ class MainWindow():
         self.preview_changes()
 
     def preview_changes(self):
-        self.renamed_uris = []
         self.infobar.hide()
-        self.rename_button.set_sensitive(True)
+
+        # Adjust scope first if necessary
+        any_dirs = False
+        iter = self.model.get_iter_first()
+        while iter != None:
+            try:
+                file_obj = self.model.get_value(iter, COL_FILE)
+                any_dirs = file_obj.is_a_dir() or any_dirs
+                iter = self.model.iter_next(iter)
+            except:
+                pass
+
+        combo = self.builder.get_object("combo_scope")
+
+        if any_dirs:
+            # on_scope_changed will update self.scope
+            combo.set_active_id(SCOPE_ALL)
+            combo.set_sensitive(False)
+        else:
+            combo.set_sensitive(True)
+
+        self.renamed_uris = []
+        any_changes = False
+
         iter = self.model.get_iter_first()
         index = 1
         while iter != None:
             try:
                 file_obj = self.model.get_value(iter, COL_FILE)
-                name = self.model.get_value(iter, COL_NAME)
-                name, ext = os.path.splitext(name)
+                orig_name = self.model.get_value(iter, COL_NAME)
+                name, ext = os.path.splitext(orig_name)
                 if ext and ext.startswith('.'):
                     ext = ext[1:]
                 if self.scope == SCOPE_NAME_ONLY:
@@ -526,9 +578,14 @@ class MainWindow():
                 elif self.scope == SCOPE_EXTENSION_ONLY:
                     ext = self.operation_function(index, ext)
                 else:
-                    name = self.operation_function(index, name)
-                    ext = self.operation_function(index, ext)
-                new_name = name + ('.' if ext else '') + ext
+                    full_name = self.operation_function(index, orig_name)
+                if self.scope == SCOPE_ALL:
+                    new_name = full_name
+                else:
+                    new_name = name + ('.' if ext else '') + ext
+
+                any_changes = (new_name != orig_name) or any_changes
+
                 self.model.set_value(iter, COL_NEW_NAME, new_name)
                 renamed_uri = file_obj.get_pending_uri(new_name)
                 if renamed_uri in self.renamed_uris:
@@ -548,6 +605,7 @@ class MainWindow():
                 index += 1
             except Exception as e:
                 print(e)
+        self.rename_button.set_sensitive(any_changes)
 
     def replace_text(self, index, string):
         case = self.replace_case_check.get_active()
